#!/usr/bin/env python
#
# Copyright (c) 2016-2017, Parallels International GmbH
#
# This file is part of disp-helper. It's free software; you can redistribute
# it and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the License,
# or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# Our contact details: Parallels International GmbH, Vordergasse 59, 8200
# Schaffhausen, Switzerland.
#

import os
import re
from datetime import datetime
from infra_cep_helper import CEPCollecting


class Static(CEPCollecting):
    def prepare_lsmod_info(self):
        out = self.execute_shell("/usr/sbin/lsmod")
        for l in out.splitlines():
            self.result["KernelModules"].insert(0, l.split(' ', 1)[0])

    def prepare_sysctl_info(self):
        out = self.execute_shell("/usr/sbin/sysctl -a")
        regex = re.compile("([^=\s]+)\s=\s(.*)")
        for l in out.splitlines():
            (name, value) = re.findall(regex, l)[0]
            self.result["Sysctl"][name] = value

    def add_ip_info(self):
        try:
            self.result["ip_a"] = self.execute_shell("/usr/sbin/ip a").strip()
        except:
            pass

    def add_top_info(self):
        try:
            self.result["top"] = self.execute_shell("/usr/bin/top -b -n1").strip()
        except:
            pass

    def prepare_uname_info(self):
        self.result["Uname"] = self.execute_shell("/usr/bin/uname -a").strip()

    def prepare_uptime_info(self):
        self.result["Uptime"] = self.execute_shell("/usr/bin/uptime").strip()

    def add_vz_conf(self):
        regex = re.compile("^([^=\#]+)=([^\#]*)$")
        d = dict()
        with open("/etc/vz/vz.conf", "r") as f:
            for l in f.readlines():
                try:
                    (name, value) = re.findall(regex, l.strip())[0]
                    d[name] = value.strip("\"")
                except:
                    continue
        self.result["VZConf"] = d

    def add_pfcache_conf(self):
        d = dict()
        if not os.path.isfile("/etc/vz/pfcache.conf"):
            return

        regex = re.compile("^([^=\#]+)=([^\#]*)$")
        with open("/etc/vz/pfcache.conf", "r") as f:
            for l in f.readlines():
                try:
                    (name, value) = re.findall(regex, l.strip())[0]
                    d[name] = value.strip("\"")
                except:
                    continue
        self.result["Pfcache"] = d

    def add_vcmmd_conf(self):
        with open("/etc/vz/vcmmd.conf", "r") as f:
            self.result["VCMMDConf"] = f.read()

    def add_session_info(self):
        self.result["Emergency"] = {"OOMs": [], "Crashes": []}
        self.result["Reboots"] = list()
        self.result["Shutdowns"] = list()
        with open("/var/log/messages", "r") as f:
            for l in f.readlines():
                if "out of memory" in l:
                    self.result["Emergency"]["OOMs"].insert(0, l.strip())

        out = self.execute_shell("/usr/bin/last -x")
        for l in out.splitlines():
            if "crash" in l:
                self.result["Emergency"]["Crashes"].insert(0, l.strip())
            elif "reboot" in l:
                self.result["Reboots"].insert(0, l.strip())
            elif "shutdown" in l:
                self.result["Shutdowns"].insert(0, l.strip())

    def prepare_kpatch_info(self):
        try:
            d = {"Installed": [], "Modules": []}
            for patch in os.listdir("/sys/kernel/kpatch/patches"):
                with open(os.path.join("/sys/kernel/kpatch/patches",
                          patch, "enabled")) as f:
                    s = {"Patch": patch, "enabled": f.read().split()}
                    d["Modules"].insert(0, s)
            for dir in os.listdir("/var/lib/kpatch"):
                s = {"Kernel": dir,
                     "Patches": [os.path.splitext(tmp)[0] for tmp
                                 in os.listdir(os.path.join("/var/lib/kpatch",
                                                            dir))]}
                d["Installed"].insert(0, s)
        except:
            d = "Unavailable"
        self.result["kpatch"] = d

    def add_openvz_switch_info(self):
        p = "/var/log/do-upgrade-vz7.log"
        s = 0
        if os.path.isfile(p):
            s = datetime.fromtimestamp(os.path.getmtime(p))
        self.result["OpenVZ2Vz7Switch"] = str(s)

    def add_vz6_switch_info(self):
        p = "/var/log/vzupgrade.log"
        s = 0
        if os.path.isfile(p):
            s = datetime.fromtimestamp(os.path.getmtime(p))
        self.result["Vz6Switch"] = str(s)

    def add_anaconda_details(self):
        d = {"GraphicInstaller": 0}
        if os.path.isfile("/var/log/anaconda/X.log"):
            d["GraphicInstaller"] = 1
        d["KernelParams"] = []
        p = "/var/log/anaconda/syslog"
        if os.path.isfile(p):
            regex = re.compile(".*kernel.*ommand line.*")
            with open(p, "r") as f:
                for line in f.readlines():
                    if regex.match(line):
                        d["KernelParams"].append(line)
                    if len(d["KernelParams"]) == 2:
                        break
        self.result["Anaconda"] = d

    def add_vzplatform_details(self):
        try:
            with open("/etc/kolla/hosts", "r") as f:
                hosts = f.read()
                hosts = hosts.strip()[1:-1]
                hosts = hosts.split(',')
                for i, s in enumerate(hosts):
                    hosts[i] = s.strip()[1:-1]
                self.result["VzPlatformHosts"] = hosts
        except:
            pass

    def add_build_info(self):
        self.result["VirtuozzoBuild"] = {
            "Release": "Unknown",
            "Version": "Unknown"
        }
        try:
            if os.path.exists("/etc/virtuozzo-release"):
                with open("/etc/virtuozzo-release", "r") as f:
                    build = f.read().replace('\n', '')
                    regex = re.compile("(.+)\s\((\d+)\)")
                    matches = re.findall(regex, build)
                    self.result["VirtuozzoBuild"]["Release"] = matches[0][0]
                    self.result["VirtuozzoBuild"]["Version"] = matches[0][1]
            elif os.path.exists("/etc/hci-release"):
                with open("/etc/hci-release", "r") as f:
                    build = f.read().replace('\n', '')
                    regex = re.compile("(.+)\s\((\d+)\)")
                    matches = re.findall(regex, build)
                    self.result["ProductBuild"]["Release"] = matches[0][0]
                    self.result["ProductBuild"]["Version"] = matches[0][1]
        except:
            pass

    def fill_results(self):
        self.result = {"KernelModules": [], "Sysctl": {}}
        self.prepare_lsmod_info()
        self.prepare_sysctl_info()
        self.prepare_uname_info()
        self.prepare_uptime_info()
        self.add_vz_conf()
        self.add_vcmmd_conf()
        self.add_session_info()
        self.prepare_kpatch_info()
        self.add_openvz_switch_info()
        self.add_anaconda_details()
        self.add_vzplatform_details()
        self.add_vz6_switch_info()
        self.add_pfcache_conf()
        self.add_build_info()
        self.add_ip_info()
        self.add_top_info()


def main():
    o = Static()
    o.run()


if __name__ == "__main__":
    main()
